# Copyright 2004 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.internet import reactor
from twisted.protocols.jabber import component, jid
from twisted.internet import task
from twisted.xish.domish import Element

import xdb
import session
import jabw
import disco
import register
import utils
import config
import debug



class PyTransport(component.Service):
	def __init__(self):
		debug.log("PyTransport: Service starting up")
		self.xdb = xdb.XDB(config.jid, mangle=True)
		self.registermanager = register.RegisterManager(self)
		
		# Discovery, as well as some builtin features
		self.discovery = disco.Discovery(self)
		self.discovery.addIdentity("gateway", "msn", config.name)
		self.discovery.addIdentity("conference", "text", config.name + " Chatrooms")
		self.discovery.addFeature("jabber:iq:register", self.registermanager.incomingRegisterIq)
		self.discovery.addFeature("http://jabber.org/protocol/muc", None)
		
		self.xmlstream = None
		self.sessions = {}
		
		# Groupchat ID handling
		self.lastID = 0
		self.reservedIDs = []
		
		self.loopCheckSessions = task.LoopingCall(self.loopCheckSessionsCall)
		self.loopCheckSessions.start(60.0) # call every ten seconds
		
		# Display active sessions if debug mode is on
		if(config.debugOn):
			self.loop = task.LoopingCall(self.loopCall)
			self.loop.start(60.0) # call every ten seconds
		
	
	def removeMe(self):
		debug.log("PyTransport: Service shutting down")
		dic = utils.copyDict(self.sessions)
		for session in dic:
			dic[session].removeMe()
	
	def makeID(self):
		newID = "r" + str(self.lastID)
		self.lastID += 1
		if(self.reservedIDs.count(newID) > 0):
			# Ack, it's already used.. Try again
			return self.makeID()
		else:
			return newID
	
	def reserveID(self, ID):
		self.reservedIDs.append(ID)
	
	def loopCall(self):
		if(len(self.sessions) > 0):
			debug.log("Sessions:")
			for key in self.sessions:
				debug.log("\t" + self.sessions[key].jabberID)
	
	def loopCheckSessionsCall(self):
		if(len(self.sessions) > 0):
			oldDict = utils.copyDict(self.sessions)
			self.sessions = {}
			for key in oldDict:
				session = oldDict[key]
				if(session.legacycon == None or session.legacycon.notificationProtocol == None or session.pytrans == None):
					debug.log("Ghost session %s found. This shouldn't happen. Trace" % (session.jabberID))
				else:
					self.sessions[key] = session
	
	def componentConnected(self, xmlstream):
		debug.log("PyTransport: Connected to main Jabberd server")
		self.xmlstream = xmlstream
		self.xmlstream.addObserver("/iq", self.discovery.onIq)
		self.xmlstream.addObserver("/presence", self.onPresence)
		self.xmlstream.addObserver("/message", self.onMessage)
	
	def componentDisconnected(self):
		debug.log("PyTransport: Disconnected from main Jabberd server")
		self.xmlstream = None
	
	def onMessage(self, el):
		fro = el.getAttribute("from")
		froj = jid.JID(fro.lower())
		if(self.sessions.has_key(froj.userhost())):
			self.sessions[froj.userhost()].onMessage(el)
		else:
			debug.log("PyTrans: Sending error response to a message outside of session.")
			reply = el
			reply.swapAttributeValues("to", "from")
			reply.attributes["type"] = "error"
			error = reply.addElement("error")
			error.attributes["type"] = "auth"
			forbidden = error.addElement("forbidden")
			forbidden.attributes["xmlns"] = "urn:ietf:params:xml:ns:xmpp-stanzas"
			text = error.addElement("text")
			text.attributes["xmlns"] = "urn:ietf:params:xml:ns:xmpp-stanzas"
			text.addContent("Error. You must log into the transport before sending messages.")
			
			self.send(reply)
	
	def onPresence(self, el):
		fro = el.getAttribute("from")
		froj = jid.JID(fro.lower())
		to = el.getAttribute("to")
		if(self.sessions.has_key(froj.userhost())):
			self.sessions[froj.userhost()].onPresence(el)
		else:
			if(to.find('@') < 0):
				# If the presence packet is to the transport (not a user) and there isn't already a session
				if(el.getAttribute("type") in [None, ""]): # Don't create a session unless they're sending available presence
					debug.log("PyTransport: Attempting to create a new session \"%s\"" % (froj.userhost()))
					s = session.makeSession(self, froj.userhost())
					if(s):
						self.sessions[froj.userhost()] = s
						debug.log("PyTransport: New session created \"%s\"" % (froj.userhost()))
						# Hack for first presence to get through
						s.onPresence(el)
					else:
						debug.log("PyTransport: Failed to create session \"%s\"" % (froj.userhost()))
						text = "Sorry. You do not appear to be registered with this transport. Please register and try again. If you are having trouble register please contact your Jabber administrator."
						jabw.sendMessage(self, to=froj.userhost(), fro=config.jid, body=text)
				
				elif(el.getAttribute("type") != "error"):
					debug.log("PyTransport: Sending unavailable presence to non-logged in user \"%s\"" % (froj.userhost()))
					el.swapAttributeValues("from", "to")
					el.attributes["type"] = "unavailable"
					self.send(el)
					return
	


class App:
	def __init__(self):
		self.c = component.buildServiceManager(config.jid, config.secret, "tcp:%s:%s" % (config.mainServer, config.port))
		self.transportSvc = PyTransport()
		self.transportSvc.setServiceParent(self.c)
		self.c.startService()
		reactor.addSystemEventTrigger('before', 'shutdown', self.shuttingDown)
	
	def shuttingDown(self):
		self.transportSvc.removeMe()



if(__name__ == "__main__"):
	app = App()
	reactor.run()

